// Class to hold the system state
// SystemState.h

#ifndef _SYSTEMSTATE_h
#define _SYSTEMSTATE_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "arduino.h"
#else
#include "WProgram.h"
#endif
#include "HPController.h"

constexpr auto  ON = 0;
constexpr auto  OFF = 1;
constexpr auto  AUTO = 2;
constexpr auto  COOL = 0;
constexpr auto  HEAT = 1;

#define DEFROST_OFFSET 30

//Modes of operation
#define MODE_RUN			0
#define MODE_WPS			1
#define MODE_SETUP			2
#define SETUP_NONE			50
#define LCDTIMER 60						//Timeout to turn off LCD backlight

//Non volatile memory storage
constexpr auto E_INIT = 500;								//EEPROM initialised flag
constexpr auto E_STARTDELAY = 0;							//Startup delay time
constexpr auto E_DEFROSTTIME = E_STARTDELAY + 4;			//Defrost time
constexpr auto E_DEFROSTTEMP = E_DEFROSTTIME + 4;			//Defrost start temperature
constexpr auto E_OPERATINGMODE = E_DEFROSTTEMP + 4;			//Operating mode
constexpr auto E_ONOFF = E_OPERATINGMODE + 4;				//Machine on/off
constexpr auto E_DEFROSTMAXTEMP = E_ONOFF + 4;				//Temperature to stop defrost (overrides time for defrost cycle)
constexpr auto E_MAXCOMPTEMP = E_DEFROSTMAXTEMP + 4;		//Maximum compressor temperature
constexpr auto E_LASTSYSTEMSTATE = E_MAXCOMPTEMP + 4;		//Last state - on or off
constexpr auto E_LASTSYSTEMMODE = E_LASTSYSTEMSTATE + 4;	//Last operating mode (heat/cool)
constexpr auto E_HEATTEMP = E_LASTSYSTEMMODE + 4;			//Temperature for heating
constexpr auto E_COOLTEMP = E_HEATTEMP + 4;					//Temperature for cooling
constexpr auto E_HYSTERISIS = E_COOLTEMP + 4;				//Hysterisis setting
constexpr auto E_DEFROSTDELAY = E_HYSTERISIS + 4;			//Time from defrost temperature to actual defrost
constexpr auto E_LASTCOMPRUN = E_DEFROSTDELAY + 4;			//Time last compressor run - note 4 bytes!
constexpr auto E_USER = E_LASTCOMPRUN + 4;					//Username
constexpr auto E_PASS = E_USER + 20;						//Password
constexpr auto E_THERMOSTATID = E_PASS + 20;				//Thermostat ID
constexpr auto E_WATCHDOG = E_THERMOSTATID + 4;				//  *** NOT USED Set to true if there was a reset by software watchdog
constexpr auto E_RUNPUMPWHENOFF = E_WATCHDOG + 4;			//Run circulating pump when not hp running
constexpr auto E_ROOMTEMP = E_RUNPUMPWHENOFF + 4;			//Desired room temperature
constexpr auto E_IP0 = E_ROOMTEMP + 4;						//Thermostat IP address
constexpr auto E_IP1 = E_IP0 + 4;							//Thermostat IP address
constexpr auto E_IP2 = E_IP1 + 4;							//Thermostat IP address
constexpr auto E_IP3 = E_IP2 + 4;							//Thermostat IP address
constexpr auto E_USETHERMOSTAT = E_IP3 + 4;					//Boolean to use thermostat
constexpr auto E_STARTTIME = E_USETHERMOSTAT + 4;			//Start time
constexpr auto E_STOPTIME = E_STARTTIME + 4;				//Stop time
constexpr auto E_AUTOMATIC = E_STOPTIME + 4;				//Automatic mode
constexpr auto E_HOSTNAME = E_AUTOMATIC + 4;				//8 bytes
constexpr auto E_ADCOFFSET = E_HOSTNAME + 8;				//1 byte
constexpr auto E_RUNDAYS = E_ADCOFFSET + 1;
constexpr auto E_RUNSTART1 = E_RUNDAYS + 2;
constexpr auto E_RUNSTOP1 = E_RUNSTART1 + 2;
constexpr auto E_RUNSTART2 = E_RUNSTOP1 + 2;
constexpr auto E_RUNSTOP2 = E_RUNSTART2 + 2;
constexpr auto E_RUNSTART3 = E_RUNSTOP2 + 2;
constexpr auto E_RUNSTOP3 = E_RUNSTART3 + 2;
constexpr auto E_TIMEBBETWEENDEFROSTS = E_RUNSTOP3 + 2;		//Minimum time between successive defrosts
constexpr auto E_NIGHTREDUCTION = E_TIMEBBETWEENDEFROSTS + 2;	//Nighttime temperature reduction
constexpr auto E_NIGHTHOUR = E_NIGHTREDUCTION + 2;			//Night hour
constexpr auto E_DAYHOUR = E_NIGHTHOUR + 2;					//Day hour
constexpr auto E_USEWEATHER = E_DAYHOUR + 2;				//Use predictive weather

//Network settings
constexpr auto E_DHCP = E_USEWEATHER + 2;
constexpr auto E_IPADD = E_DHCP + 1;
constexpr auto E_GATEWAY = E_IPADD + 4;
constexpr auto E_DNS1 = E_GATEWAY + 4;
constexpr auto E_DNS2 = E_DNS1 + 4;
constexpr auto E_MASK = E_DNS2 + 4;
constexpr auto E_SSID = E_MASK + 20;
constexpr auto E_WIFIPASS = E_SSID + 20;
constexpr auto E_TOPIC = E_WIFIPASS + 20;					//MQTT Topic
constexpr auto E_NEXT = E_TOPIC + 20;						// 


namespace {
	const char* oState[] = { "<span style='color:green'>On</span>", "<span style='color:red'>Off</span>", "<span style='color:yellow'>Auto</span>" };
	const char* oMode[] = { "<span style='color:blue'>Cool</span>", "<span style='color:red'>Heat</span>" };
	//const char* oState[] = { "On", "Off", "auto" };
	//const char* oMode[] = { "Cool", "Heat" };
}

extern AsyncWebSocket ws; // Web socket
extern PangolinMQTT mqttClient;

class SystemState
{
public:
	SystemState();
	String JsonToSave = "";
	char www_username[20];
	char www_password[20];
	char ssid[20];
	char wifipass[20];
	char hostName[8];
	bool startupDelay = false;				//Startup delay on power failure
	bool pressureSensors;					//True if the BMP sensors are found. Suppresses sending of invalid json information and trying to read the sensors
											
	//Status of individual relays.
	bool compressorState = OFF;				//Compressor relay
	bool fanState = OFF;					//Fan relay
	bool valveState = OFF;					//Valve relay
	bool pumpState = OFF;					//Pump relay
	bool bRunPumpWhenOff = true;			//Run circulation pump when hp not running

	int8_t actualWaterTemperature;
	int8_t roomHumidity;
	int8_t roomDewpoint;
	bool currentOperatingState;				//Last operating state. If running then we need a delay on startup
	bool newOperatingState;					//New operating state (for automatic)
	bool operatingMode;
	bool masterOnOff;						//Machine on/off (On = 0 (False), Off = 1 (true))
	bool overheat = false;		
	bool justStarted = true;				//Flag an on/off operation so ignore hysterisis
	bool timeSynchronized = false;			//Flag if time synchronized. If not retry 


	//Settings for thermostat
	IPAddress thermostatIP();
	int8_t thermostatTemperature = 0;
	int8_t setRoomTemperature = 21;			//Desired water temperature
	bool tsMode = MODE_RUN;
	bool bThermostatRead = false;			//Flag if thermostat read or not
	bool useThermostat = false;				//Set true if system has a thermostat
	int8_t thermostatID;					//Thermostat ID

	//System settings
	int8_t heatT;							//Heating temperature
	int8_t coolT;							//Cooling temperature
	int8_t hysterisis;						//Hysterisis
	int8_t maxCompTemperature;				//Max compressor temperature
	int8_t targetT;							//Target water temperature. Varies depending on weather (if predictive setting used)
	bool refreshData = true;				//Flag to trigger sending status json
	int8_t adcOffset = 0;					//ADC correction

	bool automatic = false;					//Auto function with timers
	int8_t startTime;
	int8_t stopTime;						//Timer for on/off scheduling

	bool bUseWeather = false;				//Use predictive heating 
	int8_t dayHour = 6;						//Time day starts
	int8_t nightHour = 21;					//Time night starts
	int8_t nightTemperature = 4;			//Amount to reduce temperature overnight

	void setOperatingState(bool state);
	void readSettings();
	void resetParameters();
	void debugOutParameters();
	String buildSettingsJson();
	void getTargetTemperature();				//Get the target temperature for the water
	String getSystemStatus(bool bWeb);
	void sendSystemStatus();					//Send a JSON with the system status
	void saveSettings();
	void expanderWrite(byte data);
	void setRelay(int relay, bool setting);
	void sendWsMessage(String message, String key);

	//SNTP clock
	boolean syncEventTriggered = false;			// True if a time even has been triggered

	bool bValveOff = false;
	long millisNow;
	long last_loop;

	// MQTT Stuff
	bool mqttActive = false;
	char topic[20];
	char topicr[21];

	//Flags to run code
	bool bMain;									//Main state machine
	bool shouldReboot = false;					//True to reboot system
	bool bGetWeather = false;					//True to read weather
	bool bSaveNetSettings = false;
	bool bSaveSettings = false;

	void saveNetSettings();
	bool getTimeBetween(int hourNow, int start, int stop);
	void setAuto(bool setting);
	void powerOnOff();
	void turnOff();
	String buildNetSettingsJson();

private:
};
extern SystemState ss;

#endif